/****************************************************************************
 * LDSBVDD.C                                                                *
 * Application: LDSBVDD.COM - DOS16 program to load SBVDD.DLL               *
 * Purpose:     Loads the sound blaster emulation VDD and then execs a DOS  *
 *              program with the emulator active                            *
 *                                                                          *
 * Copyright (c)1998 Lawrence Rust                                          *
 * Created by Lawrence Rust, Software Systems Consultants, Cambridge, UK.   *
 * lvr@softsystem.co.uk. Tel/Fax +44 1223 862391                            *
 *                                                                          *
 * Runtime:     Windows NT 4.0                                              *
 *                                                                          *
 * Build:       Microsoft Visual C++ 1.52c and SDK                          *
 *__________________________________________________________________________*
 *                                                                          *
 * Revision History:                                                        *
 *                                                                          *
 * No.   Date     By   Reason                                               *
 *--------------------------------------------------------------------------*
 * 100 01 Oct 98  lvr  Created                                              *
 * 101 15 Oct 98  lvr  Added -F option to set FM cache timeout              *
 * 102 21 Oct 98  lvr  Added -B option to set sound buffering               *
 * 103 08 Jun 99  lvr  Added -J option to en/disable joystick               *
 * 104 12 Jun 99  lvr  Added -E option to set EMU8000 base                  *
 * 105 21 Mar 00  lvr  Added -0 option to force OPL3 emulation              *
 *__________________________________________________________________________*/

#define VERSION 1.05
#define kszInfo "SoundFX 2000 sound card emulator DOS mode loader"
#define kszCopyright "Copyright (C)1998,2000 Lawrence Rust.  All rights reserved"

/* Compilation options */
#pragma warning ( disable : 4704) /* in-line assembler precludes global optimizations */


/* Exports */


/* Imports */
/* ANSI */
#include <stdlib.h>
#include <limits.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
/* DOS */
#include <process.h>
#include <conio.h>
/* App */
#include "isvbop.h"             /* NT DDK, DOS16 VDD support */


/*
 * Macro definitions
 */
#define lengthof(_a) (sizeof(_a) / sizeof(_a[0]))

#ifndef CONTAINING_RECORD
 #define CONTAINING_RECORD( ptr, type, item) \
   (type*)( (char*)(ptr) - (char*)(&((type*)0)->item) )
#endif

#define _STR(s) #s
#define STRING(s) _STR(s)


/***************************************************************************
 * Data types
 ***************************************************************************/


/***************************************************************************
 * Function prototypes
 ***************************************************************************/

static void ScanBlaster(
  const char* pszEnv,
  unsigned* puBase,
  unsigned* puIrq,
  unsigned* puDma8,
  unsigned* puDma16,
  unsigned* puMidi,
  unsigned* puEmu8000,
  unsigned* puType
);

static unsigned short LoadVdd(      /* Returns Vdd handle, 0= error */
  char const __far* lpszModule,     /* Required */
  char const __far* lpszInit,       /* Optional */
  char const __far* lpszDispatch,   /* Required */
  unsigned short unAX,              /* AX for init */
  unsigned short unDX               /* DX for init */
);


/***************************************************************************
 * Module data
 ***************************************************************************/
const static char kszCopyrightNotice[] = kszInfo " Version " STRING( VERSION) ".  " kszCopyright;
#if 0
const static char kbPacking[ 512] = "";
#endif


extern int main( int iArgc, char* aszArgv[])
  {
  const static char kszModule[] = "sbvdd.dll";
  const static char kszBlaster[] = "BLASTER";
  unsigned short unHandle;
  unsigned uBase = 0;
  unsigned uIrq = 0;
  unsigned uDma8 = 0;
  unsigned uDma16 = 0;
  unsigned uType = 0;
  unsigned uMidi = 0;
  unsigned uEmu8000 = 0;
  short nJoyPort = -1;
  short nTrace = -1;
  short nCacheTimeout = -1;
  short nDspLatency = -1;
  short nEmuCacheTimeout = -1;
  short nOpl3 = -1;
  char* pszEnv;
  int iIsNt;
  int iRet;

  /* Get default BLASTER settings */ 
  pszEnv = getenv( kszBlaster);
  if ( NULL != pszEnv)
    {
    ScanBlaster( pszEnv, &uBase, &uIrq, &uDma8, &uDma16, &uMidi, &uEmu8000, &uType);

    /* Set AWE32 if EMU8000 base set */
    if ( 0 != uEmu8000) switch ( uType)
      {
    case 0:
    case 6:
      uType = 7;
      break;
    default:
      uEmu8000 = 0;
      break;
      }
    }
    
  /* Examine args */
  while ( iArgc > 1)
    {
    unsigned long ul;
    char* pszEnd;
    char* pszArg = aszArgv[1];
    if ( '-' != *pszArg && '/' != *pszArg)
      break;

    ++pszArg;      
    switch ( *(pszArg++))
      {
    case 'h':
    case '?':
      printf(
        kszInfo ". Version " STRING( VERSION) "\n"
        "-Ax Set base address (hex)\n"
        "-In Set interrupt n\n"
        "-Dn Set 8-bit DMA channel\n"
        "-Hn Set 16-bit DMA channel\n"
        "-Pn Set MIDI port base address (hex)\n"
        "-En Set EMU8000 base address (hex)\n"
        "-Tn Set sound card, 7=AWE, 6=SB16, 4=SBProII, 3=SB2, 2=SBPro, 1=SB1.5\n"
        "-Jn Set joystick port (hex), 0= disable\n"
        "-Ln Set log detail level 0..9\n"
        "-Fn Set FM synth cache timeout ms, 0= disable\n"
        "-Bn Set sound buffer ms\n"
        "-Mn Set EMU8000 synth cache timeout ms, 0= disable\n"
        "-On Set FM synth mode: 0= device driver, 1= direct I/O, 2= emulate, 5000..44100 emulation sps\n"
      );
      exit( EXIT_SUCCESS);
      break;
 
    case 'A':
    case 'a':
      ul = strtoul( pszArg, &pszEnd, 16);
      if ( 0 < ul && ul < 0x400)
        uBase = (unsigned)ul;
      else
        {
        printf( "Invalid base address %s\n", pszArg);
        exit( EXIT_FAILURE);
        }
      break;

    case 'B':
    case 'b':
      ul = strtoul( pszArg, &pszEnd, 10);
      if ( pszEnd != pszArg && ul <= SHRT_MAX)
        nDspLatency = (short)ul;
      else
        {
        printf( "Invalid sound buffer setting %s\n", pszArg);
        exit( EXIT_FAILURE);
        }
      break;
      
    case 'D':
    case 'd':
      ul = strtoul( pszArg, &pszEnd, 10);
      if ( 0 < ul && ul < 8)
        uDma8 = (unsigned)ul;
      else
        {
        printf( "Invalid 8-bit DMA channel setting %s\n", pszArg);
        exit( EXIT_FAILURE);
        }
      break;

    case 'E':
    case 'e':
      ul = strtoul( pszArg, &pszEnd, 16);
      if ( 0 < ul && ul < 0x1000)
        {
        switch ( uType)
          {
        case 0:
        case 6:
          /* Select AWE32 */
          uType = 7;
        case 7:
          uEmu8000 = (unsigned)ul;
          break;
        default:
          printf( "EMU8000 only available on AWE32 and later\n");
          exit( EXIT_FAILURE);
          break;
          }
        }
      else
        {
        printf( "Invalid EMU8000 base address setting %s\n", pszArg);
        exit( EXIT_FAILURE);
        }
      break;

    case 'F':
    case 'f':
      ul = strtoul( pszArg, &pszEnd, 10);
      if ( pszEnd != pszArg && ul <= SHRT_MAX)
        nCacheTimeout = (short)ul;
      else
        {
        printf( "Invalid FM cache timeout setting %s\n", pszArg);
        exit( EXIT_FAILURE);
        }
      break;
      
    case 'H':
      ul = strtoul( pszArg, &pszEnd, 10);
      if ( 0 < ul && ul < 8)
        {
        switch ( uType)
          {
        case 0:
          /* Select SB16 */
          uType = 6;
        case 6:
        case 7:
          uDma16 = (unsigned)ul;
          break;
        default:
          printf( "16-bit DMA only available with SB16 and later\n");
          exit( EXIT_FAILURE);
          break;
          }
        }
      else
        {
        printf( "Invalid 16-bit DMA channel setting %s\n", pszArg);
        exit( EXIT_FAILURE);
        }
      break;

    case 'I':
    case 'i':
      ul = strtoul( pszArg, &pszEnd, 10);
      if ( 0 < ul && ul < 16)
        uIrq = (unsigned)ul;
      else
        {
        printf( "Invalid IRQ setting %s\n", pszArg);
        exit( EXIT_FAILURE);
        }
      break;

    case 'J':
    case 'j':
      ul = strtoul( pszArg, &pszEnd, 16);
      if ( pszEnd != pszArg && ul < 0x1000)
        nJoyPort = (short)ul;
      else
        {
        printf( "Invalid games port base address %s\n", pszArg);
        exit( EXIT_FAILURE);
        }
      break;
      
    case 'L':
    case 'l':
      ul = strtoul( pszArg, &pszEnd, 10);
      if ( pszEnd != pszArg && ul <= SHRT_MAX)
        nTrace = (short)ul;
      else
        {
        printf( "Invalid log detail setting %s\n", pszArg);
        exit( EXIT_FAILURE);
        }
      break;
      
    case 'M':
    case 'm':
      ul = strtoul( pszArg, &pszEnd, 10);
      if ( pszEnd != pszArg && ul <= SHRT_MAX)
        nEmuCacheTimeout = (short)ul;
      else
        {
        printf( "Invalid EMU8000 cache timeout setting %s\n", pszArg);
        exit( EXIT_FAILURE);
        }
      break;
      
    case 'O':
    case 'o':
      /* OPL3 emulation */
      ul = strtoul( pszArg, &pszEnd, 10);
      if ( pszEnd != pszArg && ul <= USHRT_MAX)
        nOpl3 = (short)ul;
      else
        nOpl3 = 2; /* Force emulation */
      break;
      
    case 'P':
    case 'p':
      ul = strtoul( pszArg, &pszEnd, 16);
      if ( 0 < ul && ul < 0x400)
        {
        switch ( uType)
          {
        case 0:
          /* Select SB16 */
          uType = 6;
        case 6:
        case 7:
          uMidi = (unsigned)ul;
          break;
        default:
          printf( "MIDI only available on SB16 and later\n");
          exit( EXIT_FAILURE);
          break;
          }
        }
      else
        {
        printf( "Invalid MIDI base address setting %s\n", pszArg);
        exit( EXIT_FAILURE);
        }
      break;

    case 'T':
    case 't':
      ul = strtoul( pszArg, &pszEnd, 10);
      switch ( ul)
        {
      case 1:
      case 2:
      case 3:
      case 4:
        if ( uEmu8000 || uMidi || uDma16)
          {
          printf( "Warning incompatible switch settings for card type %s\n", pszArg);
          uEmu8000 = uMidi = uDma16 = 0;
          }
      case 6:
      case 7:
        uType = (unsigned)ul;
        break;
      default:
        printf( "Invalid sound card type %s\n", pszArg);
        exit( EXIT_FAILURE);
        break;
        }
      break;
    
    default:
      printf( "Invalid switch '%s'\n", aszArgv[1]);
      exit( EXIT_FAILURE);
      }
      
    --iArgc;
    ++aszArgv;
    }

  /* Check we're running on NT */
  iIsNt = NULL != getenv( "PROCESSOR_ARCHITECTURE");
  if ( iIsNt)
    {
    char szBlaster[64], *psz;
    
    /* Load the soundblaster emulation VDD */
    unHandle = LoadVdd( kszModule, "VDDRegister", "VDDDispatch", 1, nTrace);
    if ( 0 == unHandle)
      {
      printf( "*** ERROR *** Failed to load NT sound emulator\n"
              "  Ensure '%s' is in the local directory or on your PATH\n", kszModule);
      return 1;
      }

    /* Pass config options to VDD */
    _asm mov cx, [uBase]
    _asm mov dx, 0x5b01
    _asm mov ax, [unHandle]
    DispatchCall()
    _asm jc L1
    _asm mov [uBase], cx
L1:
    _asm mov cx, [uIrq]
    _asm mov dx, 0x5b02
    _asm mov ax, [unHandle]
    DispatchCall()
    _asm jc L2
    _asm mov [uIrq], cx
L2:
    _asm mov cx, [uDma8]
    _asm mov dx, 0x5b03
    _asm mov ax, [unHandle]
    DispatchCall()
    _asm jc L3
    _asm mov [uDma8], cx
L3:
    _asm mov cx, [uDma16]
    _asm mov dx, 0x5b04
    _asm mov ax, [unHandle]
    DispatchCall()
    _asm jc L4
    _asm mov [uDma16], cx
L4:
    _asm mov cx, [uMidi]
    _asm mov dx, 0x5b05
    _asm mov ax, [unHandle]
    DispatchCall()
    _asm jc L5
    _asm mov [uMidi], cx
L5:
    _asm mov cx, [uType]
    _asm mov dx, 0x5b06
    _asm mov ax, [unHandle]
    DispatchCall()
    _asm jc L6
    _asm mov [uType], cx
L6:
    if ( nCacheTimeout >= 0)
      {
      _asm mov cx, [nCacheTimeout]
      _asm mov dx, 0x5b07
      _asm mov ax, [unHandle]
      DispatchCall()
      }

    if ( nDspLatency >= 0)
      {
      _asm mov cx, [nDspLatency]
      _asm mov dx, 0x5b08
      _asm mov ax, [unHandle]
      DispatchCall()
      }

    if ( nJoyPort >= 0)
      {
      _asm mov cx, [nJoyPort]
      _asm mov dx, 0x5b09
      _asm mov ax, [unHandle]
      DispatchCall()
      }

    if ( 0 != uEmu8000)
      {
      _asm mov cx, [uEmu8000]
      _asm mov dx, 0x5b0a
      _asm mov ax, [unHandle]
      DispatchCall()
      }
      
    if ( nEmuCacheTimeout >= 0)
      {
      _asm mov cx, [nEmuCacheTimeout]
      _asm mov dx, 0x5b0b
      _asm mov ax, [unHandle]
      DispatchCall()
      }

    if ( nOpl3 != -1)
      {
      _asm mov cx, [nOpl3]
      _asm mov dx, 0x5b0c
      _asm mov ax, [unHandle]
      DispatchCall()
      }

    /* Setup a BLASTER environment for the app */
    psz += sprintf( psz = szBlaster, "BLASTER=A%X I%u D%u",
      uBase ? uBase : 0x220, uIrq ? uIrq : 5, uDma8 ? uDma8 : 1
    );
    if ( uType >= 6)
      psz += sprintf( psz, " H%u P%X", uDma16 ? uDma16 : 5, uMidi ? uMidi : 0x330);
    if ( uType >= 7)
      psz += sprintf( psz, " E%X", uEmu8000 ? uEmu8000 : 0x620);
    sprintf( psz, " T%u", (uType < 6) ? uType : 6);
    _putenv( szBlaster);

    /* Config complete */
    _asm mov dx, 0x5bff
    _asm mov ax, [unHandle]
    DispatchCall()
    }

  if ( iArgc > 1)
    {
#ifndef _WINDOWS
    /* Execute the command tail */
    aszArgv[ iArgc] = NULL;
    iRet = _execvp( aszArgv[1], aszArgv + 1);

    /* Wouldn't normally be here if exec succeeded so display error */
    perror( aszArgv[1]);
#else
    /* QuickWin */
    iRet = 0;
#endif
    if ( iIsNt)
      {
      /* Unload the VDD */
      _asm mov ax, [unHandle]
      UnRegisterModule();
      }
    }
  else
    {
    /* Leave the VDD loaded */
    if ( iIsNt)
      printf( "SoundFX - NT DOS box sound card emulation resident\n");
    iRet = 0;
    }

  return iRet;
  }
 

/*
 * Parse the BLASTER environment
 */
static void ScanBlaster(
  const char* pszEnv,
  unsigned* puBase,
  unsigned* puIrq,
  unsigned* puDma8,
  unsigned* puDma16,
  unsigned* puMidi,
  unsigned* puEmu8000,
  unsigned* puType
) {
  char* psz = (char*)pszEnv;
  char c;

  while ( '\0' != (c = *psz++))
    {
    unsigned long ul;

    switch ( c)
      {
    case 'A':
      ul = strtoul( psz, &psz, 16);
      if ( 0 < ul && ul < 0x400)
        *puBase = (unsigned)ul;
      break;

    case 'I':
      ul = strtoul( psz, &psz, 10);
      if ( 0 < ul & ul < 16)
        *puIrq = (unsigned)ul;
      break;

    case 'D':
      ul = strtoul( psz, &psz, 10);
      if ( 0 < ul && ul < 8)
        *puDma8 = (unsigned)ul;
      break;

    case 'H':
      ul = strtoul( psz, &psz, 10);
      if ( 0 < ul && ul < 8)
        *puDma16 = (unsigned)ul;
      break;

    case 'T':
      ul = strtoul( psz, &psz, 10);
      switch ( ul)
        {
      case 1:
      case 2:
      case 3:
      case 4:
      case 6:
        *puType = (unsigned)ul;
        break;
        }
      break;

    case 'P':
      ul = strtoul( psz, &psz, 16);
      if ( 0 < ul && ul < 0xffff)
        *puMidi = (unsigned)ul;
      break;

    case 'E':
      ul = strtoul( psz, &psz, 16);
      if ( 0 < ul && ul <= 0x7fc)
        *puEmu8000 = (unsigned)ul;
      break;
      }

    while ( !isspace( *psz) && '\0' != *psz)
      ++psz;

    while ( isspace( *psz))
      ++psz;
    }
  }


/*
 * Load a VDD
 */
static unsigned short LoadVdd(
  char const __far* lpszModule,   /* Required */
  char const __far* lpszInit,     /* Optional */
  char const __far* lpszDispatch, /* Required */
  unsigned short unAX,
  unsigned short unDX
) {
  unsigned short unHandle;
 
  _asm push ds
  _asm push si
  _asm push di

  /* ds:si -> module name */
  _asm mov si, [WORD ptr lpszModule]

  /* es:di -> initialisation fn. name, 0:0 => none */
  _asm les di, [lpszInit]

  /* ds:bx -> dispatch fn. name */
  _asm lds bx, [lpszDispatch]

  /* Load the module, ax & dx are passed to any init fn. */
  _asm mov dx, [unDX]
  _asm mov ax, [unAX]
  RegisterModule();

  _asm pop di
  _asm pop si
  _asm pop ds

  _asm mov [unHandle], ax
  _asm jc error
  
  return unHandle;

error:
  return 0;
  }
 
/* End of file */
