/* Setup application for SoundFX */
#define PRODUCT_NAME "SoundFX 2000"
#define PRODUCT_INF ".\\soundfx.inf"
#define PRODUCT_CPL "soundfx.cpl"

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

static int OsNotSupported( void);
static BOOL IsAdmin( void);
static int Install( void);
static DWORD DisplayError( void);


extern int WINAPI WinMain(
  HINSTANCE /*hInst*/,
  HINSTANCE /*hPrevInst*/,
  LPSTR /*lpszLine*/,
  int /*nShow*/
) {
  static const char kszInfo[] = "Click OK to install " PRODUCT_NAME " on to your computer.\n"
    "Click Cancel to exit.";

  // Check OS version
  OSVERSIONINFO verinfo;
  verinfo.dwOSVersionInfoSize = sizeof( verinfo);
  GetVersionEx( &verinfo);

  int iRet = 0;
  switch ( verinfo.dwPlatformId)
    {
  case VER_PLATFORM_WIN32_NT:
    if ( verinfo.dwMajorVersion < 4 )
      return OsNotSupported(); // NT 3.51 or earlier not supported

    // Ensure admin privilege
    if ( !IsAdmin() )
      return 1;

    // Prompt to install
    if ( IDOK != MessageBox( NULL, kszInfo, PRODUCT_NAME " setup", MB_SETFOREGROUND | MB_TASKMODAL | MB_ICONQUESTION | MB_OKCANCEL))
      return 1;

    iRet = Install();
    break;

  case VER_PLATFORM_WIN32_WINDOWS:
    return OsNotSupported(); // not supported
    break;

  default:
    // Win32s not supported
    return OsNotSupported();
    }

  ExitProcess( iRet); // This is needed otherwise main hangs
  }


/*
 * Installation
 */
static int Install( void)
  {
  STARTUPINFO start;
  start.cb = sizeof( start);
  GetStartupInfo( &start);

  PROCESS_INFORMATION info;
  if ( !CreateProcess(
      NULL,
      "rundll32 syssetup,SetupInfObjectInstallAction DefaultInstall 128 " PRODUCT_INF,
      NULL,
      NULL,
      FALSE,
      0,
      NULL,
      NULL,
      &start,
      &info
    )
  ) {
    return DisplayError();
    }
  WaitForSingleObject( info.hProcess, INFINITE);

  // Run the control panel
  if ( !CreateProcess(
      NULL,
      "rundll32 shell32,Control_RunDLL " PRODUCT_CPL,
      NULL,
      NULL,
      FALSE,
      0,
      NULL,
      NULL,
      &start,
      &info
    )
  ) {
    return DisplayError();
    }

  return 0;
  }


/*
 * Display text to current error
 */
static DWORD DisplayError( void)
  {
  DWORD Err = GetLastError();
  LPVOID lpMessageBuffer = NULL;
    
  if ( FormatMessage( 
    FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
    NULL, 
    Err,  
    MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT),
    (LPTSTR) &lpMessageBuffer,  
    0,  
    NULL
    )
  ) {
    MessageBox(
      NULL,                           // HWND
      (TCHAR*)lpMessageBuffer,
      NULL,
      MB_SETFOREGROUND | MB_TASKMODAL | MB_ICONSTOP | MB_OK
    );
    if ( lpMessageBuffer)
      LocalFree( lpMessageBuffer );
    }
  else
    {
    MessageBox(
      NULL,                           // HWND
      "Error",
      NULL,
      MB_SETFOREGROUND | MB_TASKMODAL | MB_ICONSTOP | MB_OK
    );
    }

    
  SetLastError(Err);    
  return Err;
  }

  
/*
 * OS not supported
 */
static int OsNotSupported( void)
  {
  MessageBox(
    NULL,
    "This program requires Windows NT4, 2000, XP and later",
    NULL,
    MB_SETFOREGROUND | MB_TASKMODAL | MB_ICONSTOP | MB_OK
  );
  return 1;
  }


/*
 * Check for admin privileges
 */
static BOOL IsAdmin( void)
  {
#if 1
  // Check to see if we have privilege to create a service
  SC_HANDLE schSCManager;
  schSCManager = OpenSCManager( NULL, NULL, SC_MANAGER_CREATE_SERVICE);
  if ( NULL == schSCManager)
    {
    static const char kszText[] = "You must be logged on to an account with administrative privilege "
      "in order to install " PRODUCT_NAME ".";
    MessageBox( NULL, kszText, NULL, MB_SETFOREGROUND | MB_TASKMODAL | MB_ICONSTOP | MB_OK);
    return FALSE;
    }
  CloseServiceHandle( schSCManager);

  return TRUE;
#else

  typedef SC_HANDLE WINAPI FOpenSCManager( LPCSTR, LPCSTR, DWORD);
  FOpenSCManager* pfnOpenSCManager;
  typedef BOOL WINAPI FCloseServiceHandle( SC_HANDLE);
  FCloseServiceHandle* pfnCloseServiceHandle;

  // Dynamically load service control entry points
  HMODULE hmod = LoadLibrary( "advapi32.dll");
  if ( NULL == hmod
    || 0 == (pfnOpenSCManager = (FOpenSCManager*)GetProcAddress( hmod, "OpenSCManagerA"))
    || 0 == (pfnCloseServiceHandle = (FCloseServiceHandle*)GetProcAddress( hmod, "CloseServiceHandle"))
  ) {
    MessageBox(
      NULL,
      "Missing or incompatible advapi32.dll",
      NULL,
      MB_SETFOREGROUND | MB_TASKMODAL | MB_ICONSTOP | MB_OK
    );
    return FALSE;
    }

  // Check to see if we have privilege to create a service
  SC_HANDLE schSCManager;
  schSCManager = (*pfnOpenSCManager)( NULL, NULL, SC_MANAGER_CREATE_SERVICE);
  if ( NULL == schSCManager)
    {
    MessageBox(
      NULL,
      "You must be logged on to an account with administrative privilege in order to install " PRODUCT_NAME ".",
      NULL,
      MB_SETFOREGROUND | MB_TASKMODAL | MB_ICONSTOP | MB_OK
    );
    return FALSE;
    }

  (*pfnCloseServiceHandle)( schSCManager);

  return TRUE;
#endif
  }

/* End of file */
