/****************************************************************************
 *                                                                          *
 * portctl.h                                                                *
 * --------------                                                           *
 *                                                                          *
 * Definitions of the private IOCTL codes implemented by the portio driver. *
 *                                                                          *
 * Copyright (c)1999 Lawrence Rust                                          *
 * Created by Lawrence Rust, Software Systems Consultants, Cambridge, UK.   *
 * lvr@softsystem.co.uk. Tel/Fax +44 1223 862391                            *
 *__________________________________________________________________________*
 *                                                                          *
 * Revision History:                                                        *
 *                                                                          *
 * No.   Date     By   Reason                                               *
 *--------------------------------------------------------------------------*
 * 100  08 Jun 99  lvr  Creation                                            *
 *__________________________________________________________________________*/

#ifndef PORTCTL_H
#define PORTCTL_H 0x100

/* Dependencies */
#ifdef _NTDDK_
#include <devioctl.h>
#else
#include <winioctl.h>
#endif

/* Device type */
#define PORTIO_DEVICE_NAME  "portio" /* in \??\ */
#define PORTIO_TYPE     0x9999  /* Microsoft uses 0 - 0x7FFF, OEMs use 0x8000 - 0xFFFF */
#define PORTIO_FCMIN    0x900   /* Microsoft uses function codes 0-0x7FF, OEM's use 0x800 - 0xFFF */

#define PORTIO_READ8 \
   CTL_CODE( PORTIO_TYPE, (PORTIO_FCMIN+0x00), METHOD_BUFFERED, FILE_READ_ACCESS)
/* lpvInBuffer -> WORD array of ports */
/* lpvOutBuffer -> BYTE array of data read */

#define PORTIO_READ16 \
   CTL_CODE( PORTIO_TYPE, (PORTIO_FCMIN+0x01), METHOD_BUFFERED, FILE_READ_ACCESS)
/* lpvInBuffer -> WORD array of ports */
/* lpvOutBuffer -> WORD array of data read */

#define PORTIO_WRITE8 \
   CTL_CODE( PORTIO_TYPE, (PORTIO_FCMIN+0x02), METHOD_BUFFERED, FILE_WRITE_ACCESS)
/* lpvInBuffer -> Array of PORTIO_SWrite */

#define PORTIO_WRITE16 \
   CTL_CODE( PORTIO_TYPE, (PORTIO_FCMIN+0x03), METHOD_BUFFERED, FILE_WRITE_ACCESS)
/* lpvInBuffer -> Array of PORTIO_SWrite */


/*
 * IOCTL API - Input and output buffer structures
 */
typedef struct PORTIO_SWrite
  {
  USHORT wRegister;                     // IN: Register offset 0..
  union
    {
    USHORT wData;
    UCHAR ubData;
    } u;                                  // I/O: data written/read
  } PORTIO_SWrite;

#endif /* ndef PORTCTL_H */
/* End of file */
