/****************************************************************************
 * DSP.H                                                                    *
 * Interface to the DSP emulator                                            *
 *                                                                          *
 * Copyright (c)1998 Lawrence Rust                                          *
 * Created by Lawrence Rust, Software Systems Consultants, Cambridge, UK.   *
 * lvr@softsystem.co.uk. Tel/Fax +44 1223 862391                            *
 *__________________________________________________________________________*
 *                                                                          *
 * Revision History:                                                        *
 *                                                                          *
 * No.   Date     By   Reason                                               *
 *--------------------------------------------------------------------------*
 * 100 02 Oct 98  lvr  Created                                              *
 * 101 11 May 99  lvr  Added DspRestartOutput                               *
 * 102 21 Mar 00  lvr  Added DspSuspend/Resume                              *
 *__________________________________________________________________________*/
#ifndef DSP_H
#define DSP_H 0x102

/* Dependencies */
#pragma warning ( disable : 4201) /* nonstandard extension used : nameless struct/union */
#pragma warning ( disable : 4214) /* nonstandard extension used : bit field types other than int */
#include <windows.h>
#pragma warning ( default : 4201)
#pragma warning ( default : 4214)

/* Types */
struct SSbvdd;
typedef struct SDsp SDsp;

/* Functions */
extern SDsp* DspInit( const struct SSbvdd*);
extern void DspFinal( register SDsp*);
extern void DspTerminate( register SDsp*);
extern void __fastcall DspReset( register SDsp*, unsigned);
extern BYTE __fastcall DspRead( register SDsp*);
extern void __fastcall DspWrite( register SDsp*, BYTE);
extern BYTE __fastcall DspReadStatus( register SDsp*);
extern BYTE __fastcall DspWriteStatus( register SDsp*);
extern BYTE __fastcall DspIrqStatus( register SDsp*);
extern void __fastcall DspIack( register SDsp*, unsigned);
extern BOOL __fastcall DspInterrupt( register SDsp*, unsigned);
extern BOOL __fastcall DspRestartOutput( register SDsp*);

extern void DspSetWaveOutputInterval( register SDsp*, unsigned);
extern BOOL DspSetConfig(
  register SDsp*,
  BYTE ubType,
  BYTE ubIrq,
  BYTE ubDma8,
  BYTE ubDma16
);
extern BYTE __fastcall DspGetType( register SDsp*);
extern BYTE __fastcall DspGetIrq( register SDsp*);
extern BYTE __fastcall DspGetDma8( register SDsp*);
extern BYTE __fastcall DspGetDma16( register SDsp*);

extern BOOL DspSetVolume(
  register SDsp*, 
  BYTE ubLeftVolume,
  BYTE ubRightVolume
);

extern void DspSuspend( register SDsp*);
extern void DspResume( register SDsp*);

#endif /* ndef DSP_H */
/* End of file */
