/****************************************************************************
 * MIDI.H                                                                   *
 * Interface to the MPU401 MIDI emulator                                    *
 *                                                                          *
 * Copyright (c)1998 Lawrence Rust                                          *
 * Created by Lawrence Rust, Software Systems Consultants, Cambridge, UK.   *
 * lvr@softsystem.co.uk. Tel/Fax +44 1223 862391                            *
 *__________________________________________________________________________*
 *                                                                          *
 * Revision History:                                                        *
 *                                                                          *
 * No.   Date     By   Reason                                               *
 *--------------------------------------------------------------------------*
 * 100 02 Oct 98  lvr  Created                                              *
 * 101 03 Feb 00  lvr  Added MidiOpen                                       *
 * 102 22 Mar 00  lvr  Added EMidiMode to MidiOpen                          *
 *__________________________________________________________________________*/
#ifndef MIDI_H
#define MIDI_H 0x102

/* Dependencies */
#pragma warning ( disable : 4201) /* nonstandard extension used : nameless struct/union */
#pragma warning ( disable : 4214) /* nonstandard extension used : bit field types other than int */
#include <windows.h>
#pragma warning ( default : 4201)
#pragma warning ( default : 4214)

/* Types */
struct SSbvdd;
typedef struct SMidi SMidi;

typedef enum EMidiMode
  {
  kMidiOut,
  kMidiInOut,
  kMidiInOutIrq
  } EMidiMode;

/* Functions */
extern SMidi* MidiInit( const struct SSbvdd*);
extern void MidiFinal( register SMidi*);
extern void MidiTerminate( register SMidi*);
extern BYTE __fastcall MidiRead( register SMidi*);
extern void __fastcall MidiWrite( register SMidi*, BYTE);
extern BYTE __fastcall MidiStatus( register SMidi*);
extern void __fastcall MidiCommand( register SMidi*, BYTE);
extern BOOL MidiOpen( register SMidi* pMidi, EMidiMode);

extern BOOL __fastcall MidiInDataReady( register SMidi*);
extern unsigned __fastcall MidiInData( register SMidi*, BYTE aubData[], DWORD* pdwTime);

#endif /* ndef MIDI_H */
/* End of file */
