/****************************************************************************
 * SBVDD.H                                                                  *
 * Common interfaces to SBVDD.DLL                                           *
 *                                                                          *
 * Copyright (c)1998 Lawrence Rust                                          *
 * Created by Lawrence Rust, Software Systems Consultants, Cambridge, UK.   *
 * lvr@softsystem.co.uk. Tel/Fax +44 1223 862391                            *
 *__________________________________________________________________________*
 *                                                                          *
 * Revision History:                                                        *
 *                                                                          *
 * No.   Date     By   Reason                                               *
 *--------------------------------------------------------------------------*
 * 100 02 Oct 98  lvr  Created                                              *
 * 101 06 Nov 98  lvr  Add offsetof                                         *
 * 102 28 Apr 99  lvr  Add joystick emulation                               *
 * 103 20 Mar 00  lvr  DirectSound integration                              *
 * 104 19 Feb 01  lvr  #ifndef added to lengthof & offsetof                 *
 *__________________________________________________________________________*/
#ifndef SBVDD_H
#define SBVDD_H 0x104

/* Dependencies */
#pragma warning ( disable : 4201) /* nonstandard extension used : nameless struct/union */
#pragma warning ( disable : 4214) /* nonstandard extension used : bit field types other than int */
#include <windows.h>
#include <mmsystem.h>                   // WAVEFORMATEX
#pragma warning ( default : 4201)
#pragma warning ( default : 4214)

// NT DDK
#ifdef __cplusplus
extern "C" {
#endif
#include <vddsvc.h>                     // VDD services
#ifdef __cplusplus
}
#endif


/*
 * Macros
 */
#ifndef lengthof
 #define lengthof(_a) (sizeof(_a) / sizeof((_a)[0]))
#endif
#ifndef offsetof
 #define offsetof( type, field) ((size_t)&(((type*)0)->field))
#endif

#ifndef CONTAINING_RECORD
 #define CONTAINING_RECORD( ptr, type, item) \
   (type*)( (char*)(ptr) - offsetof( type, item) )
#endif

/* Function called by ASSERT to abort */
#define ALERT_TERMINATE() ExitProcess( 1)

#ifdef __cplusplus
 #define EXPORT extern "C" __declspec( dllexport)
#else
 #define EXPORT extern __declspec( dllexport)
#endif


/*
 * Constants
 */
#if OPT_EVALUATION
 #define kszAppName "SoundFX 2000 EVALUATION"
#else
 #define kszAppName "SoundFX 2000"
#endif

/*
 * Data types
 */
/* Forward refs */
struct SDsp;
struct SMixer;
struct SFm;
struct SMidi;
#if OPT_JOYPORT
struct SGame;
#endif
#if OPT_EMU8000
struct SEmu8000;
#endif
struct SDs;

/* Emulator instance data */
typedef struct SSbvdd
  {
  struct SDsp* pDsp;
  struct SMixer* pMixer;
  struct SFm* pFm;
  struct SMidi* pMidi;
#if OPT_JOYPORT
  struct SGame* pGame;
#endif
#if OPT_EMU8000
  struct SEmu8000* pEmu8000;
#endif
  struct SDs* pDs;
  } SSbvdd;


/*
 * Global data
 */
extern HANDLE g_hVDD;                   /* VDD module handle */
extern HANDLE g_hVddHeap;               /* VDD local heap */


/*
 * Functions
 */
extern BOOL SoundFxInit( void);
extern void SoundFxFinal( void);
extern BOOL SoundFxDispatch( USHORT unCmd, WORD wParam);
extern VOID SoundFxCreateProcess( USHORT DosPDB);

/* Configuration data access */
extern BOOL ReadRegValue(
  const char* pszValue,                 /* IN: value name */
  DWORD dwTypeRequired,                 /* IN: value type, REG_DWORD,REG_BINARY,REG_SZ  etc */
  size_t size,                          /* IN: data buffer size */
  void* pvData                          /* OUT: data */
);

/* Start a service */
extern BOOL StartAService(
  const char* pszService                /* IN: Name of service to start */
);

#if OPT_EVALUATION
 /* Start evaluation timeout */
 extern void StartEvalTimeout( void);
#define START_EVAL_TIMEOUT() do { \
   static void (* const pfn)( void) = &StartEvalTimeout; \
   (*pfn)(); \
 } while ( 0)
#else
 #define START_EVAL_TIMEOUT() ((void)0)
#endif

#if defined DEBUG || defined _DEBUG || DBG
 /* Dump CPU state to log */
 extern int DumpState( int);
#endif

#if 1 /* TODO: Undoc'd NTVDM */
#ifdef __cplusplus
extern "C" {
#endif
enum EBtn
  {
  kBtnOk = 1,
  kBtnCancel = 2,
  kBtnYes = 3,
  kBtnNo = 4,
  kBtnRetry = 5,
  kBtnAbort = 6,
  kBtnIgnore = 7,
  kBtnClose = 8,
  kBtnHelp = 9
  };
extern ULONG WOWSysErrorBox(
  LPCSTR lpszTitle,
  LPCSTR lpszText,
  ULONG ulBtn1,
  ULONG ulBtn2,
  ULONG ulBtn3
);
extern void SuspendTimerThread( void);
extern void ResumeTimerThread( void);
extern void DispatchInterrupts( void);
extern ULONG* getIntelRegistersPointer( void);
#ifdef __cplusplus
}
#endif
#endif

#endif /* ndef SBVDD_H */
/* End of file */
