@echo off
rem This batch file searches for and removes instruction sequences in DOS mode
rem programs that disable interrupts.
rem
rem Under Windows NT the VDM maintains a virtual interrupt enable state for
rem each DOS box.  Execution of an CLI or STI instruction causes a trap to the
rem emulator which then updates the virtual interrupt state.  Many programs
rem save the interrupt state before disabling them by executing a pushf
rem instruction.  Unfortunately due to a limitation in the Intel architecture
rem the popf instruction that would normally restore the interrupt state does
rem not cause a trap and hence the virtual interrupt state in the VDM gets out
rem of step with what the program expects and further virtual interrupts such
rem as timer, mouse or sound card are not delivered to the program.

setlocal
rem parse options

rem Test for aggressive option
if not "%1" == "-2" goto args
set opt=1
shift


rem validate args
:args
if "%1" == "" goto error
if "%2" == "" goto error
if "%2" == "%1" goto error

if exist %1 goto test2
echo No such file %1
goto error

:test2
if not exist %2 goto start
choice Overwrite %2
if errorlevel 2 goto end


:start
rem Replace pushfd, pop eax, cli with pushfd, pop eax, nop
@echo on
patch -f\x9c\x58\xfa -r\x9c\x58\x90 %1 %2
@echo off
if errorlevel 1 goto end


if not "%opt%" == "1" goto end
rem Replace pushfd, cli with pushfd, nop
@echo on
patch -f\x9c\xfa -r\x9c\x90 %2
@echo off

goto end


:error
echo usage %0 [-2] {file to patch} {patched file}
echo -2 option enables additional processing - recommended for DOOM 1 and 2
echo.

:end
set opt=
endlocal
